/*
 Class TReg32 for (x)Harbour
 (c) Grigory Filatov <gfilatov@freemail.ru>

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this file.

 The exception is that, if you link this code with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking  
 this code into it.

*/

#include "hbclass.ch"
#include "common.ch"

#define KEY_ALL_ACCESS		63
#define REG_SZ			1
#define REG_DWORD		4

CLASS TReg32

   DATA cRegKey, nHandle, nError, lError

   METHOD New( nKey, cRegKey, lShowError )

   METHOD Create( nKey, cRegKey, lShowError )

   METHOD Get( cSubKey, uVar )

   METHOD Set( cSubKey, uVar )

   METHOD Delete( cSubKey )

   METHOD KeyDelete( cSubKey )

   METHOD Close() BLOCK { |Self| If( ::lError, , RegCloseKey( ::nHandle ) ) }

ENDCLASS


METHOD New( nKey, cRegKey, lShowError ) CLASS TReg32

   local nReturn, nHandle := 0

   DEFAULT cRegKey TO ""

   nReturn := RegOpenKeyExA( nKey, cRegKey,, KEY_ALL_ACCESS, @nHandle )

   ::cRegKey := cRegKey
   ::nHandle := nHandle

   if nReturn != 0
      ::lError := .t.
      IF lShowError == NIL .OR. lShowError
         MsgStop( "Error creating TReg32 object ("+LTrim(Str(nReturn))+")" )
      ENDIF
   else
      ::lError := .f.
   endif

return Self


METHOD Create( nKey, cRegKey, lShowError ) CLASS TReg32

   local nReturn, nHandle := 0

   DEFAULT cRegKey TO ""

   nReturn := RegCreateKey( nKey, cRegKey, @nHandle )

   if nReturn != 0
      ::lError := .t.
      IF lShowError == NIL .OR. lShowError
         MsgStop("Error creating TReg32 object ("+LTrim(Str(nReturn))+")" )
      ENDIF
   else
      ::lError := .f.
   endif

   ::nError := RegOpenKeyExA( nKey, cRegKey,, KEY_ALL_ACCESS, @nHandle )

   ::cRegKey := cRegKey
   ::nHandle := nHandle

return Self


METHOD Get( cSubkey, uVar ) CLASS TReg32

   local cValue := ""
   local nType := 0
   local nLen := 0
   local cType

   if !::lError

      cType := ValType( uVar )

      if cType == 'N'
         nType := REG_DWORD
         nLen  := 4
         cValue := L2Bin( uVar )
      endif

      ::nError = RegQueryValueExA( ::nHandle, cSubkey, 0, @nType, @cValue, @nLen )
      uVar := cValue

      do case
         case cType == "N"
              uVar := Bin2L( uVar )

         case cType == "D"
              uVar := CToD( uVar )

         case cType == "L"
              uVar := ( Upper( uVar ) == ".T." )

         case cType == "N"
              uVar := Val( uVar )
      endcase

   endif

return uVar


METHOD Set( cSubKey, uVar ) CLASS TReg32

   local nType, nLen, cType

   if !::lError

      cType := ValType( uVar )

      if cType == 'N'
         nType := REG_DWORD
         nLen  := 4
         uVar  := L2Bin( uVar )
      else
         nType := REG_SZ
         do case
            case cType == "D"
                 uVar := DToC( uVar )

            case cType == "L"
                 uVar := If( uVar, ".T.", ".F." )
         endcase
         nLen := Len( uVar )
      endif
      ::nError := RegSetValueExA( ::nHandle, cSubkey, 0, nType, @uVar, nLen )

   endif

return nil


METHOD Delete( cSubKey ) CLASS TReg32

   if !::lError

      ::nError := RegDeleteValueA( ::nHandle, cSubkey )

   endif

return nil


METHOD KeyDelete( cSubKey ) CLASS TReg32

   if !::lError

      ::nError := RegDeleteKey( ::nHandle, cSubkey )

   endif

return nil
